'''Tools for l10n.'''
from os import system, makedirs, remove
from os.path import exists
from shutil import move, copy
from ya2.build.build import files


class LanguageBuilder:
    '''Tools for building files for l10n.'''

    @staticmethod
    def mo(tgt, lng_dir_code, appname):
        '''Builds the mo file in the lng_dir_code directory.'''
        lng_code = tgt[len(lng_dir_code):].split('/')[0]
        lng_dir = lng_dir_code + lng_code + '/LC_MESSAGES/'
        cmd = 'msgfmt -o {lng_dir}{appname}.mo assets/locale/po/{lng_code}.po'
        system(cmd.format(lng_dir=lng_dir, appname=appname, lng_code=lng_code))

    @staticmethod
    def pot(appname, pot_path):
        '''Builds the pot file in the lng_dir_code directory.'''
        src_files = ' '.join(files(['py']))
        cmd_tmpl = 'xgettext -ci18n -d {appname} -L python ' + \
            '-o {pot_path}{appname}.pot '
        system(cmd_tmpl.format(appname=appname, pot_path=pot_path) + src_files)

    @staticmethod
    def merge(lng_code, tgt_path, lng_dir, appname):
        '''Merges the new strings with the previous ones.'''
        lng_base_dir = LanguageBuilder.__prepare(lng_dir, lng_code, appname)
        LanguageBuilder.__merge(lng_base_dir, lng_code, appname, tgt_path)
        LanguageBuilder.__postprocess(lng_code)

    @staticmethod
    def __prepare(lng_base_dir, lng, appname):
        '''Prepares a directory for working with languages.'''
        makedirs(lng_base_dir + lng + '/LC_MESSAGES', exist_ok=True)
        lng_dir = lng_base_dir + lng + '/LC_MESSAGES/'
        if not exists('assets/locale/po/' + lng + '.po'):
            lines_to_fix = ['CHARSET/UTF-8', 'ENCODING/8bit']
            [LanguageBuilder.__fix_line(line, lng_dir, appname) for line in lines_to_fix]
            copy(lng_dir + appname + '.pot', lng_dir + appname + '.po')
        return lng_dir

    @staticmethod
    def __fix_line(line, lng_dir, appname):
        '''Fixes po files (misaligned entries).'''
        cmd_tmpl = "sed 's/{line}/' {lng_dir}{appname}.pot > " + \
            "{lng_dir}{appname}tmp.po"
        system(cmd_tmpl.format(line=line, lng_dir=lng_dir, appname=appname))
        move(lng_dir + appname + 'tmp.po', lng_dir + appname + '.pot')

    @staticmethod
    def __merge(lng_dir, lng_code, appname, tgt_path):
        '''Manages the msgmerge's invokation.'''
        print('merge', lng_dir)
        cmd = 'msgmerge -o {lng_dir}{appname}merge.po ' + \
            '{tgt_path}{lng_code}.po {tgt_path}{appname}.pot'
        cmd = cmd.format(lng_dir=lng_dir, lng_code=lng_code, appname=appname,
                         tgt_path=tgt_path)
        system(cmd)
        copy(lng_dir + appname + 'merge.po', 'assets/locale/po/%s.po' % lng_code)
        poname_tmpl = '{lng_dir}{appname}merge.po'
        remove(poname_tmpl.format(lng_dir=lng_dir, appname=appname))

    @staticmethod
    def __postprocess(lng_code):
        '''Fixes po files at the end of the building process.'''
        lines = open('assets/locale/po/%s.po' % lng_code, 'r').readlines()
        with open('assets/locale/po/%s.po' % lng_code, 'w') as outf:
            for line in lines:
                po_str = '"POT-Creation-Date: YEAR-MO-DA HO:MI+ZONE\\n"\n'
                outf.write(po_str if line.startswith(po_str[:20]) else line)
