from panda3d.core import GraphicsWindow, WindowProperties
from ya2.p3d.gui import P3dImg as Img
from ya2.patterns.gameobject import GameObject


class MouseCursorFacade:

    def show(self):
        if not self.eng.cfg.dev_cfg.functional_test:
            return self.cursor_img.show()
    def hide(self): return self.cursor_img.hide()


class MouseCursor(GameObject, MouseCursorFacade):

    def __init__(self, filepath, scale, color, hotspot):
        GameObject.__init__(self)
        MouseCursorFacade.__init__(self)
        if not filepath: return
        self.__set_std_cursor(False)
        self.cursor_img = Img(filepath, scale=scale, foreground=True)
        self.cursor_img.img.set_color(color)
        #if self.eng.cfg.dev_cfg.functional_test:
        #    self.cursor_img.hide()
        self.hotspot_dx = scale[0] * (1 - 2 * hotspot[0])
        self.hotspot_dy = scale[2] * (1 - 2 * hotspot[1])
        #self.eng.attach_obs(self.on_frame)
        #self.eng.attach_obs(self.on_frame_unpausable)
        self._tsk = taskMgr.add(self.__on_frame, 'on frame cursor')

    @staticmethod
    def __set_std_cursor(show):
        props = WindowProperties()
        props.set_cursor_hidden(not show)
        if isinstance(base.win, GraphicsWindow):
            base.win.requestProperties(props)

    #def show_standard(self): self.eng.lib.show_std_cursor()

    #def hide_standard(self): self.eng.lib.hide_std_cursor()

    #def cursor_top(self):
    #    self.cursor_img.reparent_to(self.cursor_img.parent)

    def __on_frame(self, task):
        mwn = base.mouseWatcherNode
        if not mwn or not mwn.hasMouse():
            return task.again
        mouse = mwn.get_mouse_x(), mwn.get_mouse_y()
        h_x = mouse[0] * base.getAspectRatio() + self.hotspot_dx
        self.cursor_img.set_pos((h_x, mouse[1] - self.hotspot_dy))
        return task.again

    #def on_frame(self):
    #    if not self.eng.pause.paused: self.__on_frame()

    #def on_frame_unpausable(self):
    #    if self.eng.pause.paused: self.__on_frame()

    def set_image(self, img):
        self.cursor_img.img.set_texture(loader.load_texture(img), 1)

    def destroy(self):
        taskMgr.remove(self._tsk)
        self.cursor_img.destroy()
